@@ -14,21 +14,19 @@ struct A1 {
     typedef T value_type;
 };
 
-#if !defined(BOOST_NO_CXX11_ALLOCATOR)
 template<class T>
 struct A2 {
     typedef T value_type;
-    typedef std::true_type propagate_on_container_swap;
+    struct propagate_on_container_swap {
+        static const bool value = true;
+    };
 };
-#endif
 
 int main()
 {
     BOOST_TEST_TRAIT_FALSE((boost::
         allocator_propagate_on_container_swap<A1<int> >::type));
-#if !defined(BOOST_NO_CXX11_ALLOCATOR)
     BOOST_TEST_TRAIT_TRUE((boost::
         allocator_propagate_on_container_swap<A2<int> >::type));
-#endif
     return boost::report_errors();
 }
