@@ -287,7 +287,7 @@ thread_context::main() {
 #elif defined(__aarch64__)
     asm(".cfi_undefined x30");
 #elif defined(__s390x__)
-    asm(".cfi_undefined r14");
+    asm(".cfi_undefined %r14");
 #else
     #warning "Backtracing from seastar threads may be broken"
 #endif
